/*
 * tbveb.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>

#include "extb.h"
#include "extb_status.h"
#include "extb_irsend.h"
#include "extb_setup_menu.h"
#include "extb_osd_menu.h"
#include "i18n.h"

static const char *VERSION        = "0.0.12";
static const char *DESCRIPTION    = "VDR Extension Board";
static const char *MAINMENUENTRY  = "Extension Board";

cExtBLircWatch *gLircWatch = NULL;

class cPluginExtB : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginExtB(void);
  virtual ~cPluginExtB();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
private:
  cExtBStatusMonitor *statusMonitor;
  };

cPluginExtB::cPluginExtB(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  statusMonitor = NULL;
}

cPluginExtB::~cPluginExtB()
{
  // Clean up after yourself!
  delete statusMonitor;
  lirc_send("M2");
  lirc_close();
}

const char *cPluginExtB::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginExtB::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginExtB::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginExtB::Start(void)
{
  RegisterI18n(Phrases);

  // Start any background activities the plugin shall perform.
  if(!lirc_open())
    {
      isyslog("lirc_open() failed!");
      return (false);
    }
  /* active RGB + audio output */
  statusMonitor = new cExtBStatusMonitor;
  gLircWatch = new cExtBLircWatch("/dev/lircd");

  return true;
}

void cPluginExtB::Housekeeping(void)
{
  //Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginExtB::MainMenuAction(void)
{
  return (new cExtBOsdMenu);
}

cMenuSetupPage *cPluginExtB::SetupMenu(void)
{
  return (new cExtBMenuSetup);
}

bool cPluginExtB::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

VDRPLUGINCREATOR(cPluginExtB); // Don't touch this!

