/* 	$Id: rc.c,v 5.1 2002/09/12 19:53:56 lirc Exp $	 */

/*
  
  rc -  application for sending IR-codes via lirc
  
  Copyright (C) 1998 Christoph Bartelmus (lirc@bartelmus.de)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/

#define LIRCD "/dev/lircd"

/* stolen from vdr tools.h */
extern int SysLogLevel;

#define esyslog(a...) void( (SysLogLevel > 0) ? syslog(LOG_ERR,   a) : void() )
#define isyslog(a...) void( (SysLogLevel > 1) ? syslog(LOG_INFO,  a) : void() )
#define dsyslog(a...) void( (SysLogLevel > 2) ? syslog(LOG_DEBUG, a) : void() )

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>
#include <syslog.h>

#include "extb_irsend.h"

#define PACKET_SIZE 256
/* three seconds */
#define TIMEOUT 3

static int timeout=0;
static int fd=-1;
static struct sigaction act;

static void sigalrm(int sig)
{
  timeout=1;
}

const char *lirc_read_string()
{
  static char buffer[PACKET_SIZE+1]="";
  char *end;
  static int ptr=0;
  ssize_t ret;

  if(-1 == fd)
    {
      lirc_open();
    }

  if(ptr>0)
    {
      memmove(buffer,buffer+ptr,strlen(buffer+ptr)+1);
      ptr=strlen(buffer);
      end=strchr(buffer,'\n');
    }
  else
    {
      end=NULL;
    }
  alarm(TIMEOUT);
  while(end==NULL)
    {
      if(PACKET_SIZE<=ptr)
	{
	  dsyslog("extb/lirc_read_string(%d): bad packet", fd);
	  ptr=0;
	  return(NULL);
	}
      ret=read(fd,buffer+ptr,PACKET_SIZE-ptr);
      
      if(ret<=0 || timeout)
	{
	  if(timeout)
	    {
	      dsyslog("extb/lirc_read_string(%d): timeout", fd);
	    }
	  else
	    {
	      alarm(0);
	    }
	  ptr=0;
	  return(NULL);
	}
      buffer[ptr+ret]=0;
      ptr=strlen(buffer);
      end=strchr(buffer,'\n');
    }
  alarm(0);
  timeout=0;
  
  end[0]=0;
  ptr=strlen(buffer)+1;

  dsyslog("extb/lirc_read_string(%d): result='%s'", fd, buffer);

  return(buffer);
}

enum packet_state
  {
    P_BEGIN,
    P_MESSAGE,
    P_STATUS,
    P_DATA,
    P_N,
    P_DATA_N,
    P_END
  };

int lirc_send_packet(const char *packet)
{
  int done,todo;
  const char *string,*data;
  char *endptr;
  enum packet_state state;
  int status;
  unsigned long n;
  unsigned long data_n=0;

  if(-1 == fd)
    {
      lirc_open();
    }

  dsyslog("lirc_send_packet(%d): %s", fd, packet);
  
  todo=strlen(packet);
  data=packet;
  while(todo>0)
    {
      done=write(fd,(void *) data,todo);
      if(done<0)
	{
	  dsyslog("extb/lirc_send_packet(%d): could not send packet: %s", fd, strerror(errno));
	  return(-1);
	}
      data+=done;
      todo-=done;
    }
  
  /* get response */
  status=0;
  state=P_BEGIN;
  n=0;
  while(1)
    {
      string=lirc_read_string();
      dsyslog("extb/lirc_send_packet/read(%d): %s", fd, string);
      if(string==NULL)
	return(-1);
      switch(state)
	{
	case P_BEGIN:
	  if(strcasecmp(string,"BEGIN")!=0)
	    {
	      continue;
	    }
	  state=P_MESSAGE;
	  break;
	case P_MESSAGE:
	  if(strncasecmp(string,packet,strlen(string))!=0 ||
	     strlen(string)+1!=strlen(packet))
	    {
	      state=P_BEGIN;
	      continue;
	    }
	  state=P_STATUS;
	  break;
	case P_STATUS:
	  if(strcasecmp(string,"SUCCESS")==0)
	    {
	      status = 0;
	    }
	  else if(strcasecmp(string,"END")==0)
	    {
	      status=0;
	      return(status);
	    }
	  else if(strcasecmp(string,"ERROR")==0)
	    {
	      dsyslog("extb/lirc_send_packet(%d): command failed: %s", fd, packet);
	      status = -1;
	    }
	  else
	    {
	      goto bad_packet;
	    }
	  state=P_DATA;
	  break;
	case P_DATA:
	  if(strcasecmp(string,"END")==0)
	    {
	      return(status);
	    }
	  else if(strcasecmp(string,"DATA")==0)
	    {
	      state=P_N;
	      break;
	    }
	  goto bad_packet;
	case P_N:
	  errno=0;
	  data_n=strtoul(string,&endptr,0);
	  if(!*string || *endptr)
	    {
	      goto bad_packet;
	    }
	  if(data_n==0)
	    {
	      state=P_END;
	    }
	  else
	    {
	      state=P_DATA_N;
	    }
	  break;
	case P_DATA_N:
	  dsyslog("extb/lirc_send_packet(%d): %s", fd, string);
	  n++;
	  if(n==data_n) state=P_END;
	  break;
	case P_END:
	  if(strcasecmp(string,"END")==0)
	    {
	      return(status);
	    }
	  goto bad_packet;
	  break;
	}
    }
 bad_packet:
  dsyslog("extb/lirc_send_packet(%d): bad return packet", fd);
  return(-1);
}

bool lirc_open()
{
  struct sockaddr_un addr;

  dsyslog("extb/lirc_open");

  act.sa_handler=sigalrm;
  sigemptyset(&act.sa_mask);
  act.sa_flags=0;           /* we need EINTR */
  sigaction(SIGALRM,&act,NULL);

  addr.sun_family=AF_UNIX;
  strcpy(addr.sun_path,LIRCD);
  fd=socket(AF_UNIX,SOCK_STREAM,0);
  if(fd==-1)
    {
      dsyslog("extb/lirc_open(%d): could not open socket: %s", fd, strerror(errno));
      return(false);
    }
  if(connect(fd,(struct sockaddr *)&addr,sizeof(addr))==-1)
    {
      dsyslog("extb/lirc_open(%d): could not connect to socket: %s", fd, strerror(errno));
      return(false);
    }
  return (true);
}

void lirc_close()
{
  dsyslog("extb/lirc_close(%d)", fd);
  close(fd);
  fd = -1;
}

void lirc_send(const char *cmd)
{
  char tmpbuff[256];

  sprintf(tmpbuff, "send_once EXTB_TX %s\n", cmd);
  if(-1 == lirc_send_packet(tmpbuff))
    {
      lirc_close();
      lirc_open();
    }
}

