#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "extb_lircwatch.h"

#define REPEATLIMIT  20 // ms
#define REPEATDELAY 350 // ms
#define KEYPRESSDELAY 150 // ms

cExtBLircWatch::cExtBLircWatch(char *DeviceName)
{
  struct sockaddr_un addr;
  addr.sun_family = AF_UNIX;
  strcpy(addr.sun_path, DeviceName);
  if ((f = socket(AF_UNIX, SOCK_STREAM, 0)) >= 0) {
     if (connect(f, (struct sockaddr *)&addr, sizeof(addr)) >= 0) {
        Start();
        return;
        }
     LOG_ERROR_STR(DeviceName);
     close(f);
     }
  else
     LOG_ERROR_STR(DeviceName);
  f = -1;
}

cExtBLircWatch::~cExtBLircWatch()
{
  Cancel();
}

void cExtBLircWatch::StartWatch()
{
  watch = true;
  watch_success = false;
  video_modes = 0;
  scart_boot_modes = 0;
  video_found = false;
  scart_boot_found = false;
}

void cExtBLircWatch::Action(void)
{
  dsyslog("LIRC remote control thread started (pid=%d)", getpid());

  int FirstTime = 0;
  int LastTime = 0;
  char buf[LIRC_BUFFER_SIZE];
  char LastKeyName[LIRC_KEY_BUF] = "";
  bool repeat = false;
  int timeout = -1;

  for (; f >= 0;) {

      LOCK_THREAD;

      bool ready = cFile::FileReady(f, timeout);
      int ret = ready ? safe_read(f, buf, sizeof(buf)) : -1;

      if (ready && ret <= 0 ) {
         esyslog("ERROR: lircd connection lost");
         close(f);
         f = -1;
         break;
         }

      if (ready && ret > 21) {
         int count;
         char KeyName[LIRC_KEY_BUF];
	 char Remote[256];
         sscanf(buf, "%*x %*x %29s %s", KeyName, Remote); // '29' in '%29s' is LIRC_KEY_BUF-1!
         int Now = time_ms();
         LastTime = Now;
         dsyslog("lirc_watch: %s / %s", KeyName, Remote);
	 if(watch && 0 == strcmp("EXTB_RX", Remote))
	   {
	     dsyslog("checking key: >%s<", KeyName);
	     if(0 == strncmp(KeyName, "V", 1))
	       {
		 dsyslog("video mode found");
		 video_modes = atoi(&KeyName[1]);
		 video_found = true;
	       }
	     if(0 == strncmp(KeyName, "SB", 2))
	       {
		 dsyslog("scart boot mode found");
		 sscanf(&KeyName[2], "%x", &scart_boot_modes);
		 scart_boot_found = true;
	       }
	     if(video_found && scart_boot_found)
	       {
		 dsyslog("answer from extb complete");
		 watch_success = true;
	       }
	   }
         }
      }
}
