#include <vdr/font.h>

#include "extb.h"
#include "i18n.h"
#include "extb_osd_menu.h"
#include "extb_irsend.h"

class cLEDItem : public cOsdItem
{
 protected:
  int min_item;
  int max_item;
  static const char* const cLEDItem::reset_cmd  = "R_LD%d";
  static const char* const cLEDItem::set_cmd    = "S_LD%d";
  static const char* const cLEDItem::toggle_cmd = "T_LD%d";
 public:
  cLEDItem(int _min_item, int _max_item, const char *name)
    {
      min_item = _min_item;
      max_item = _max_item;
      SetText(name);
    }
  virtual eOSState ProcessKey(eKeys Key);
};

eOSState cLEDItem::ProcessKey(eKeys Key)
{
  const char *buff = "";
  eOSState state = cOsdItem::ProcessKey(Key);

  if(kRed == Key)
    {
      buff = reset_cmd;
    }
  else if(kGreen == Key)
    {
      buff = set_cmd;
    }
  else if(kYellow == Key)
    {
      buff = toggle_cmd;
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  if(strlen(buff) > 0)
    {
      for(int i=min_item; i <= max_item; i++)
	{
	  char tmp[6];
	  sprintf(tmp, buff, i);
	  lirc_send(tmp);
	}
    }

  return (state);
}

class cExtOutItem : public cOsdItem
{
 protected:
  int min_item;
  int max_item;
  static const char* const cExtOutItem::reset_cmd  = "R_OUT%d";
  static const char* const cExtOutItem::set_cmd    = "S_OUT%d";
  static const char* const cExtOutItem::toggle_cmd = "T_OUT%d";
 public:
  cExtOutItem(int _min_item, int _max_item, const char *name)
    {
      min_item = _min_item;
      max_item = _max_item;
      SetText(name);
    }
  virtual eOSState ProcessKey(eKeys Key);
};

eOSState cExtOutItem::ProcessKey(eKeys Key)
{
  const char *buff = "";
  eOSState state = cOsdItem::ProcessKey(Key);

  if(kRed == Key)
    {
      buff = reset_cmd;
    }
  else if(kGreen == Key)
    {
      buff = set_cmd;
    }
  else if(kYellow == Key)
    {
      buff = toggle_cmd;
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  if(strlen(buff) > 0)
    {
      for(int i=min_item; i <= max_item; i++)
	{
	  char tmp[6];
	  sprintf(tmp, buff, i);
	  lirc_send(tmp);
	}
    }

  return (state);
}

class cStdOnOffItem : public cOsdItem
{
 protected:
  char cmd_on[10];
  char cmd_off[10];
 public:
  cStdOnOffItem(const char* const name, const char* _cmd_on, const char* _cmd_off)
    {
      SetText(name);
      strcpy(cmd_on, _cmd_on);
      strcpy(cmd_off, _cmd_off);
    }
  virtual eOSState ProcessKey(eKeys Key);
};

eOSState cStdOnOffItem::ProcessKey(eKeys Key)
{
  const char *buff = "";
  eOSState state = cOsdItem::ProcessKey(Key);

  if(kRed == Key)
    {
      buff = cmd_off;
    }
  else if(kGreen == Key)
    {
      buff = cmd_on;
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  if(strlen(buff) > 0)
    {
      lirc_send(buff);
    }

  return (state);
}

class cMacroItem : public cOsdItem
{
 protected:
  char cmd[10];
  bool reset_cmd_after_key;
  time_t reset_cmd_time;
 public:
  cMacroItem(const char* const name, const char* _cmd)
    {
      SetText(name);
      strcpy(cmd, _cmd);
      reset_cmd_after_key = false;
      reset_cmd_time = (time_t)-1;
    }
  virtual eOSState ProcessKey(eKeys Key);
};

eOSState cMacroItem::ProcessKey(eKeys Key)
{
  eOSState state = cOsdItem::ProcessKey(Key);

  if(reset_cmd_after_key)
    {
      time_t now = time(NULL);
      if(now - reset_cmd_time > 15)
	{
	  reset_cmd_after_key = false;
	  lirc_send("M1");
	}
    }

  if(kRed == Key || kGreen == Key || kOk == Key)
    {
      lirc_send(cmd);
      if(kRed == Key)
	{
	  reset_cmd_after_key = true;
	  reset_cmd_time = time(NULL);
	}
    }
  else if(kYellow == Key)
    {
      lirc_send("M1");
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  return (state);
}

class cLEDMenu : public cOsdMenu
{
 public:
  cLEDMenu(void);
  virtual eOSState ProcessKey(eKeys Key);
};

cLEDMenu::cLEDMenu()
  : cOsdMenu(tr("Extension Board - led test"), 10, 10)
{
  SetHasHotkeys();

  Add(new cLEDItem(1, 1, hk(tr("LED #1"))));
  Add(new cLEDItem(2, 2, hk(tr("LED #2"))));
  Add(new cLEDItem(3, 3, hk(tr("LED #3"))));
  Add(new cLEDItem(4, 4, hk(tr("LED #4"))));
  Add(new cLEDItem(5, 5, hk(tr("LED #5"))));
  Add(new cLEDItem(6, 6, hk(tr("LED #6"))));
  Add(new cLEDItem(1, 6, hk(tr("LED #1-6"))));
  SetHelp(tr("off"), tr("on"), tr("toggle"), tr("back"));
}

eOSState cLEDMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);
  
  return (state);
}

class cLCDMenu : public cOsdMenu
{
 public:
  cLCDMenu(void);
};

cLCDMenu::cLCDMenu()
  : cOsdMenu(tr("Extension Board - lcd test"), 10, 10)
{
  SetHasHotkeys();

  Add(new cStdOnOffItem(hk(tr("LCD contract voltage")), "S_LCD", "R_LCD"));
  Add(new cStdOnOffItem(hk(tr("LCD backlight")), "S_LCD_BL", "R_LCD_BL"));

  SetHelp(tr("off"), tr("on"), NULL, tr("back"));
}

class cExtOutMenu : public cOsdMenu
{
 public:
  cExtOutMenu(void);
};

cExtOutMenu::cExtOutMenu()
  : cOsdMenu(tr("Extension Board - external output"), 10, 10)
{
  SetHasHotkeys();

  Add(new cExtOutItem(1, 1, hk(tr("Output #1"))));
  Add(new cExtOutItem(2, 2, hk(tr("Output #2"))));
  Add(new cExtOutItem(3, 3, hk(tr("Output #3"))));
  Add(new cExtOutItem(4, 4, hk(tr("Output #4"))));
  Add(new cExtOutItem(1, 4, hk(tr("Output #1-4"))));
  SetHelp(tr("off"), tr("on"), tr("toggle"), tr("back"));
}

class cMacroMenu : public cOsdMenu
{
 public:
  cMacroMenu(void);
};

cMacroMenu::cMacroMenu()
  : cOsdMenu(tr("Extension Board - macros"), 10, 10)
{
  SetHasHotkeys();

  Add(new cMacroItem(hk(tr("Macro #1")), "M1"));
  Add(new cMacroItem(hk(tr("Macro #2")), "M2"));
  Add(new cMacroItem(hk(tr("Macro #3")), "M3"));
  Add(new cMacroItem(hk(tr("Macro #4")), "M4"));
  Add(new cMacroItem(hk(tr("Macro #5")), "M5"));
  Add(new cMacroItem(hk(tr("Macro #6")), "M6"));
  Add(new cMacroItem(hk(tr("Macro #7")), "M7"));
  Add(new cMacroItem(hk(tr("Macro #8")), "M8"));
  Add(new cMacroItem(hk(tr("Macro #9")), "M9"));
  Add(new cMacroItem(hk(tr("Macro #10")), "M10"));

  SetHelp(tr("test 15s"), tr("send"), tr("reset (M1)"), tr("back"));
}

class cVideoMenu : public cOsdMenu
{
 public:
  cVideoMenu(void);
};

cVideoMenu::cVideoMenu()
  : cOsdMenu(tr("Extension Board - video"), 10, 10)
{
  SetHasHotkeys();

  Add(new cMacroItem(hk(tr("Videoquelle DVB")), "S_DVB"));
  Add(new cMacroItem(hk(tr("Videoquelle VGA")), "S_VGA"));
  Add(new cMacroItem(hk(tr("Videoquelle AUX")), "S_AUX"));
  Add(new cMacroItem(hk(tr("Videoquelle aus")), "R_VID_ALL"));
  Add(new cMacroItem(hk(tr("Videoquelle DVB/VGA umschalten")), "T_DVB_VGA"));
  Add(new cMacroItem(hk(tr("Videoquelle DVB/AUX umschalten")), "T_DVB_AUX"));
  Add(new cMacroItem(hk(tr("Videoquelle VGA/AUX umschalten")), "T_VGA_AUX"));
  Add(new cMacroItem(hk(tr("Scart #8 6V")), "S_SC8_6V"));
  Add(new cMacroItem(hk(tr("Scart #8 12V")), "R_SC8_6V"));

  SetHelp(tr("test 15s"), tr("send"), tr("reset (M1)"), tr("back"));
}

class cAudioMenu : public cOsdMenu
{
 public:
  cAudioMenu(void);
};

cAudioMenu::cAudioMenu()
  : cOsdMenu(tr("Extension Board - audio"), 10, 10)
{
  SetHasHotkeys();

  Add(new cMacroItem(hk(tr("analog Audio an")), "S_ANA"));
  Add(new cMacroItem(hk(tr("analog Audio aus")), "R_ANA"));
  Add(new cMacroItem(hk(tr("analog Audio umschalten")), "T_ANA"));
  Add(new cMacroItem(hk(tr("digital Audio an")), "S_DIA"));
  Add(new cMacroItem(hk(tr("digital Audio aus")), "R_DIA"));
  Add(new cMacroItem(hk(tr("digital Audio umschalten")), "T_DIA"));
  Add(new cMacroItem(hk(tr("analog+digital Audio an")), "S_ANDIA"));
  Add(new cMacroItem(hk(tr("analog+digital Audio aus")), "R_ANDIA"));
  Add(new cMacroItem(hk(tr("analog+digital Audio umschalten")), "T_ANDIA"));

  SetHelp(tr("test 15s"), tr("send"), tr("reset (M1)"), tr("back"));
}

cExtBOsdMenu::cExtBOsdMenu(void)
  : cOsdMenu(tr("Extension Board"))
{
  SetHasHotkeys();

  Add(new cOsdItem(hk(tr("Status LEDs")),              osUser1));
  Add(new cOsdItem(hk(tr("LC-Display & Backlight")),   osUser2));
  Add(new cOsdItem(hk(tr("External Outputs")),         osUser3));
  Add(new cOsdItem(hk(tr("Macros")),                   osUser4));
  Add(new cOsdItem(hk(tr("Audio")),                    osUser5));
  Add(new cOsdItem(hk(tr("Video")),                    osUser6));

  SetHelp(NULL, NULL, NULL, tr("back"));
  Display();
}

cExtBOsdMenu::~cExtBOsdMenu()
{
}

eOSState cExtBOsdMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  switch(state)
    {
    case osUser1: 
      state = AddSubMenu(new cLEDMenu);
      break;
      
    case osUser2: 
      state = AddSubMenu(new cLCDMenu);
      break;
      
    case osUser3: 
      state = AddSubMenu(new cExtOutMenu);
      break;
      
    case osUser4: 
      state = AddSubMenu(new cMacroMenu);
      break;
      
    case osUser5: 
      state = AddSubMenu(new cAudioMenu);
      break;

    case osUser6: 
      state = AddSubMenu(new cVideoMenu);
      break;

    case osContinue:
      break;

    default:
      if(kBlue == Key)
	{
	  state = osBack;
	}
    }

  if(!HasSubMenu())
    Display();

  return (state);
}
